//470 - звук под пищалку с генератором
//473 - звук под пищалку без генератора

long readVcc() {
  // Read 1.1V reference against AVcc
  // set the reference to Vcc and the measurement to the internal 1.1V reference
  
  #if defined(__AVR_ATmega32U4__) || defined(__AVR_ATmega1280__) || defined(__AVR_ATmega2560__)
    ADMUX = _BV(REFS0) | _BV(MUX4) | _BV(MUX3) | _BV(MUX2) | _BV(MUX1);
  #elif defined (__AVR_ATtiny24__) || defined(__AVR_ATtiny44__) || defined(__AVR_ATtiny84__)
    ADMUX = _BV(MUX5) | _BV(MUX0);
  #elif defined (__AVR_ATtiny25__) || defined(__AVR_ATtiny45__) || defined(__AVR_ATtiny85__)
    ADMUX = _BV(MUX3) | _BV(MUX2);
  #else
    ADMUX = _BV(REFS0) | _BV(MUX3) | _BV(MUX2) | _BV(MUX1);
  #endif  
 
  delay(2); // Wait for Vref to settle
  ADCSRA |= _BV(ADSC); // Start conversion
  while (bit_is_set(ADCSRA,ADSC)); // measuring
 
  uint8_t low  = ADCL; // must read ADCL first - it then locks ADCH  
  uint8_t high = ADCH; // unlocks both
 
  long result = (high<<8) | low;
 
  result = 1125300L / result; // Calculate Vcc (in mV); 1125300 = 1.1*1023*1000
  return result; // Vcc in millivolts
  
}

//--------------- подпрограммы ---------------------------
//перезапись массива из ОЗУ в EEPROM после взлета
//byte inputBytes[32] = { 0 }; //буфер данных для записи/чтения в/из 24C256 (страница)
void MasToEE(unsigned int tAlt) {
  //заполним 8 байт данных P,X,Y,Z для записи страницы в 24с
  inputBytes[0] = lowByte(tAlt);     //P

  //в старший байт в бит 7 ставим состояние датчика вибрации (0-замкнут)
  if (digitalRead(_sw) == LOW) inputBytes[1] = 0b01111111 & highByte(tAlt);
  else inputBytes[1] = 0b10000000 | highByte(tAlt);
  //в старший байт в бит 6 ставим состояние второго датчика вибрации (0-замкнут)
  if (digitalRead(_swu) == LOW) inputBytes[1] = 0b10111111 & inputBytes[1];
  else inputBytes[1] = 0b01000000 | inputBytes[1];
//  inputBytes[1] = highByte(tAlt);
  /*
  // читаем "сырые" данные с акселерометра (двухбайтовые)
  raw = accelerometer.readRaw();
  Atmp = raw.XAxis;
  inputBytes[2] = lowByte(Atmp);   //X
  inputBytes[3] = highByte(Atmp);
  Atmp = raw.YAxis;
  inputBytes[4] = lowByte(Atmp);   //Y
  inputBytes[5] = highByte(Atmp);
  Atmp = raw.ZAxis;
  inputBytes[6] = lowByte(Atmp);   //Z
  inputBytes[7] = highByte(Atmp);
  */
  //v853 - вместо ускорения в [2,3] пишем температуру, в [6,7] напряжение
  tAlt = BMP280_readTemperature() +2000; //температура с BMP280    на 20 градусов выше и в 100 раз больше
  inputBytes[2] = lowByte(tAlt);   
  inputBytes[3] = highByte(tAlt);
  tAlt = readVcc();                //питание устройства VCC(5V)    006
  inputBytes[6] = lowByte(tAlt);   
  inputBytes[7] = highByte(tAlt);
  //запись в EEPROM штатного замера высоты и ускорений
     EEPROM.write(addr, inputBytes[0]);
     EEPROM.write(addr+1, inputBytes[1]);
//  ee24c.i2c_eeprom_write_page(EEPROM_ADDRESS, addr, inputBytes, 8);  //
  //сдвигаем адрес в цикле основной программы
//---------------------------------------------------
  //если не весь массив из ОЗУ переписан в EEPROM - потихоньку переписываем
  if (Nbuf >= 0) {
    delay(10); // пауза чтоб запись завершилась
    //еще не весь массив перенесен в EEPROM
    //заполняем страницу (8 байт)
    for(int i8=0; i8<8; i8++) {
        inputBytes[i8] = Mas[Nbuf+i8];     
    }
    //запись в EEPROM штатного замера высоты и ускорений
//    ee24c.i2c_eeprom_write_page(EEPROM_ADDRESS, Nbuf, inputBytes, 8);  //
    int Nbuf1 = Nbuf >> 2;  //временный адрес, в мегу пишем только 2 байта из 8
    EEPROM.write(Nbuf1 + N0, Mas[Nbuf]);   //одна 2х байтовая ячейка
    EEPROM.write(Nbuf1 + N0 + 1, Mas[Nbuf+1]);
    Nbuf = Nbuf - 8;  //смещаем указатель массива
  }
}

//запись отсчетов в массив в ОЗУ до старта
void AltToMas(unsigned int tAlt) {
  Mas[Nbuf] = lowByte(tAlt);     //P
  //в старший байт в бит 7 ставим состояние датчика вибрации (0-замкнут)
  if (digitalRead(_sw) == LOW) Mas[Nbuf+1] = 0b01111111 & highByte(tAlt);
  else Mas[Nbuf+1] = 0b10000000 | highByte(tAlt);
  //в старший байт в бит 6 ставим состояние второго датчика вибрации (0-замкнут)
  if (digitalRead(_swu) == LOW) Mas[Nbuf+1] = 0b10111111 & Mas[Nbuf+1];
  else Mas[Nbuf+1] = 0b01000000 | Mas[Nbuf+1];
//  Mas[Nbuf+1] = highByte(tAlt);
  /*
  // читаем "сырые" данные с акселерометра (двухбайтовые)
  raw = accelerometer.readRaw();
                Atmp = raw.XAxis;
  Mas[Nbuf+2] = lowByte(Atmp);   //X
  Mas[Nbuf+3] = highByte(Atmp);
                Atmp = raw.YAxis;
  Mas[Nbuf+4] = lowByte(Atmp);   //Y
  Mas[Nbuf+5] = highByte(Atmp);
                Atmp = raw.ZAxis;
  Mas[Nbuf+6] = lowByte(Atmp);   //Z
  Mas[Nbuf+7] = highByte(Atmp);
  */
  //v853 - вместо ускорения в [2,3] пишем температуру, в [6,7] напряжение
  tAlt = BMP280_readTemperature() +2000; //температура с BMP280    на 20 градусов выше и в 100 раз больше
  Mas[Nbuf+2] = lowByte(tAlt);   
  Mas[Nbuf+3] = highByte(tAlt);
  tAlt = readVcc();                //питание устройства VCC(5V)    006
  Mas[Nbuf+6] = lowByte(tAlt);   
  Mas[Nbuf+7] = highByte(tAlt);
  
  Nbuf = Nbuf + 8;
  if (Nbuf >= MasMax * 2 * 4) {
    //весь массив заполнен, надо сдвигать
    for (Nbuf = 2*4; Nbuf < MasMax*2*4; Nbuf ++) {
       Mas[Nbuf-2*4] = Mas[Nbuf];
    }
    Nbuf = (MasMax-1)*2*4; //ставим на последнюю ячейку
  }  
}
//--------------------------------------------------------

//считываем давление с датчика
//скорректированное давление с привязкой к начальному отсчету
unsigned int readPressure(void)
{
   unsigned long tmp_P = BMP280_readPressure();  //давление в Паскалях (101000 Pa) *256
   return tmp_P/256 - MinP; //чтоб в два байта уложилось   
}

//процедура формирования 1000Гц на выводе rf длиной 200 мс
void tone200ms() {
      for (int it = 0; it < 200; it ++) {
        digitalWrite(_rf433, HIGH);
        delayMicroseconds(500);               
        digitalWrite(_rf433, LOW); 
        delayMicroseconds(500);    
      }        
}
// процедура формирования сигнала 4КГц на 0,4 сек
void Beep() { 
  pinMode(_v_plus, OUTPUT);   

    /*
      digitalWrite(_v_plus, HIGH);  // подаем HIGH на выход  +
      delay(400);                    // ожидаем 
      digitalWrite(_v_plus, LOW);   // выключаем: LOW на выходе  +
      */

  for (int i=0; i<1600; i++) {    // циклов примерно на 0,4 сек  
      digitalWrite(_v_plus, HIGH);  // подаем HIGH на выход  +
      delayMicroseconds(128);       // ожидаем полпериода от 4КГц
      digitalWrite(_v_plus, LOW);   // устанавливаем LOW на выходе  +
      delayMicroseconds(128);       // ожидаем полпериода от 4КГц
  }
  
}

// процедура формирования сигнала 4КГц на 0,06 сек на пьезоизлучателе
void Beep60() { 
  /*
      digitalWrite(_v_plus, HIGH);  // подаем HIGH на выход  +
      delay(60);                    // ожидаем 
      digitalWrite(_v_plus, LOW);   // выключаем: LOW на выходе  +
      */
 
  for (int i=0; i<230; i++) {       // циклов примерно на 0,06 сек  
      digitalWrite(_v_plus, HIGH);  // подаем HIGH на выход  +
      delayMicroseconds(128);       // ожидаем полпериода от 4КГц
      digitalWrite(_v_plus, LOW);   // устанавливаем LOW на выходе  +
      delayMicroseconds(128);       // ожидаем полпериода от 4КГц
  }
  
}
