/*
 Альтиметр на базе Pro mini + BMP280
 непосредственное подключение, рассчитано на питание не выше 4В
 .._T_.. - версия с определением максимума и включением запала
 
Включение ВМР280 к Arduino Pro mini
6 контактов 
D4 = MISO (SDO) input
D5 = CS         out
D6 = MOSI (SDI) out
D7 = SCK        out
D8 = GND        out  (LOW)
D9 = VCC        out  (HIGH при питании 3В)
  
высотомер: пишет в EEPROM отсчеты высоты 
- если при включении неисправен BMP280 - частые мигания
- иначе - до начала фиксации данных три двойные вспышки
- определение старта по изменению высоты (давления) на  Hd от начальной
(до старта одиночные мигания 1/8   2 сек)
- после обнаружения старта светодиод помаргивает 
- после заполнения памяти светодиод гаснет
 
v.1 - только проверка замера напряжения питания 
      VCC > maxVCC - двойные вспышки; иначе мигание 1 сек
v.2 - включение BMP280 и проверка инициализации
      если не откликается - тройные вспышки
      если ОК - мигание 1 сек
v.3 - регистрация отсчетов в EEPROM
      перед началом обнаружения старта три двойные вспышки
      при обнаружении старта одиночные вспышки
      при регистрации полета - погашено
      по окончании регистрации - мигание 1 сек
v.4 - добавлена регистрация начальных параметров
      000 температура (+20 *100)
      002 напряжение VCC (в мВ)
      004 начальное значение давления (в Па - 80000) (медиана из 3х замеров)
      006 температура после заполнения памяти (+20 *100)
      008 напряжение VCC после заполнения памяти (в мВ)
     - исправлен алгоритм обнаружения старта (как в S_4)
v.6 - изменен алгоритм начала работы
     - кроме проверки напряжения питания, проверяется ячейка EEPROM (защита от перезагрузки) 1=FF
	   если FF - переходим к регистрации, иначе СТОП (двойные писки ПРД, Led погашен)
	 - введен анализ перемычки для обнуления ячейки EEPROM (чтоб можно было обойти пред.пункт)
	   наличие перемычки test между 10 и 11. если есть - сбрасываем ячейку в FF, (двойные писки ПРД, 
	   Led включен) и ждем RESET
	 - добавлена работа с передатчиком 433МГц (после окончания регистрации - тройные писки ПРД)
	 - ПРД подключается к выходу Тх (D1)  1 - включен.
v.7 - работа со звуком
     - по окончании вывода данных - писки
     - перед регистрацией один писк, если ВМР в норме то еще три
     - при обнаружении пуска короткие писки
     - после окончания регистрации - писки
     - при запрете записи - писки (пауза длиннее)
    - алгоритм определения старта усложнен до 3х превышений (см _P_6) 
v.8 - анализ включения по геркону/кнопке
     - после включения по Тх выдается 0 для удержания транзистора блока питания открытым (p-n-p в цепи VCC, 
       база на землю - включение
v.9 - развита система звуковых оповещений при работе
v.9.1 - переделан под противофазное управление пьезоизлучателем на 4КГц
v.9.2 - переделано упр. RF433, изменена звук индик при не записи
v.9.3 - звук переделан на А0 А1, RF433 на TxD, определение старта увеличено до 2м
v.12  - включение запала (тиристор на D3) после максимума
v.12a - исправлена инициализация звука 
v.12b - испр.уст.флага "полет" после сбоя при провале питания (стоп(2)) и замера питания
v.12с - флаг "полет" после сбоя при провале питания = 2 
v.12cA - упрощенная версия: под ATMega168; нет контроля игнитора; 200мс период
--------------
v.200 - тестовый вариант для проверки BMP280 (03.03.2020)
         - под 328, 100мс, запись в 328, восстановлен вывод данных СОМ при > 4В
         - ВМР280 подключена под вариант трекера (2..7) 7=VCC
         - изменены библиотеки; исправлена ошибка записи в ОЗУ первого отсчета
v.202 - тест
         - до 1 м уменьшены пороги взлета и апогея; 
         - добавлена свето индикация конца (стоп 6)
-----------------------------------
v.302 - проба варианта для РП-19Г (не проверялась)
v.303 - добавлен GPS
v.304 - продолжение GPS
v.305 - отлажен текст СМС (номер, напряжение, высота, координаты)
  305a - светодиод на пин 13 (проверка)
v.306 - изменена индикация ожидания спутников
      - исправлен getfix() - не было выхода false
      - переставлена проверка признака полета на после инициал GSM & GPS
  306a - светодиод на пин 11, коррекция СМС и считывания
  306b - в опр.старта добавлен сброс давления; 
       - после старта флаг полета в "1" (защита от сбоя питания)
       - в СМС по сбою питания "А" (обычно идет "S")
  308 - изменения в алгоритме старта и срабатывания ССР
       - в старте проверка на меньшее давление (было меньше равно)
       - в сраб ССР введено время блокировки анализа tBlokCCP (3сек)
       - добавлено сраб ССР по истечении tOnCCP (15 сек) с старта
  308a - проба на новой плате Beep(10), VD(13), BMP280
  308b - test: убран GSM и GPS (только регистрация давления) 
         (работает ОК, при отсутствии питания ВМР280, оно проходит мимо (3,1В) - ВМР работает
         при питании 5В - на ВМР280 0,2В)
         (при подключенном М590 работает с перемычкой, в комнате не берет - только на улице)
  308с - восстановлен GSM (без GPS) [в getFix блокировка GPS]
         первая СМС переставлена после фиксации давления (перед стартом)
         в СМС добавлен вывод текущего давления в КПа ("К")      ОК
  309  - исправлен вывод давления в СМС после окончания регистрации
         в СМС при вкл игитора передается миним давление altMax (а не текущее!)
         добавлена запись строки в 24сХХ после первой СМС (исх.давл. и координаты точки старта)
         включен GPS (был откл для теста в 308b и 308c)
         включен вывод 24сХХ, сокращен размер до 128 стр по 32 на 64 сек  (был откл для теста в 308b и 308c)
         снова включен анализ флага наличия полета  EEPROM.read(a_fl_fly)
         убран АТ+CLIP  (вкл АОН для вызова, т.к. вызов не проверяем)
         ------------------
         даже без спутников прорывается в начале к старту (СМС не пришла!)
         нет звуков по окончании регистрации
         повторное вкл после полета - все СМС 1 А (без спутников), когда спутники появились - ОК
  309a - NumSMS переделана на float для нормального print в СМС
         в СМС добавлен вывод игнитора и числа спутников  1.011S - включен, 11 спутников
         ужесточена проверка строки $GPGGA
  309b - первая СМС переставлена до определения исходного давления и пауза после нее 10 сек
         (иначе в 309а сильно скачет давление - излучение или просадка питания - ложный старт)
  310  - формат вывода дополнен блоком для ПО Дианко (mfr.exe)  - OK
  310a - изменено смещение (40000) для хранения давления в двухбайтовой ячейке (плохо
         -----------------------------
  400  - добавлен акселерометр ADXL345 I2C 
  400a - развитие, переделаны Did8, вывод посл.кан и проч
         данные регистрируются в 24с256 (16К (8 байт по 100мс) + 16К (32 байта по 500мс)) - примерно на 3 мин
  400b - исправление ошибок вывода
         -----------------------------
  400ng  - отключен GPS  (РП20-7)
  400ng1 - изменены временные параметры (3,5 сек блокировка ССР; 20 сек срабатывание ССР)
  450    - отключено GSM (2,5  и 15 сек)
  451    - 3,5 и 20 сек (РП20-8)
  452    - (РП-21) вариант без ADXL345, питание ВМР280 от вывода контроллера (9), откл М590; доп файл Dig852 вместо Dig8
           земля ВМР280 на контр (8); управл. транзистором на (2); динамик на А0  (под плату nRF24_6m_A_BMP_24C(v1.1))  
  453    - (РП21-5) изменены блокировки (3,5 и 20 сек), уменьшен сдвиг давления 73000 (было 80000), т.е. макс выс 2,5-3 км
           для этого введена конст MinP=73000, изменена Dig853: вместо ускХ пишем температуру; вместо ускZ = VCC
           при выводе MFR вместо общего ускорения выводим напряжение в вольтах
  454    - (РП21-6) исправлены ошибки регистрации температуры и напряжения, изменена Dig854
  455    - (РП21-7) вся 24C256 отдана под регистрацию по 8 байт (4096 отсчетов = 6,8 мин)
  460    - MinP=40000 (примерно до 7,1 км); отключена проверка корректности давления и на старте, и в полете.
           добавлен акселерометр MPU6050 (только библиотека)
  461    - прописан акселерометр MPU6050
  --------------------------------
        для РП22
  470    - версия для красн.платы, убран акселерометр, изменены контакты подкл.ВМР,MOSFET,ZUM; запись в Mega328
           уменьшены времена блокировок (2,5 сек и 15 сек) 
           изменено формирование звуков (пищалка с генератором)  Dig470  
  471    - исправлена ошибка в стартовом адресе записи данных 
  472    - исправлены ошибки вывода двухбайтовых значений давления  
  --------------------------------
        для РП25, 26, 27
  473t   - в цикле 1 сек выводит Р,Т и напряжение питания (9600); вкл транзистора и звука
           изменено формирование звуков (пищалка без генератора)  Dig473  
  473    - BMP280 переназначены выводы 4..9  
  474    - вывод дополнен расшифровкой служебных ячеек; скорость вывода уменьшена до 9600; N0 = 20
  --------------------------------
  480    - как регистратор (в РП18): добавлен контактный датчик (ст.бит в отсчете), период регистрации 50мс
  481    - добавлен второй конт датчик D14 в отсчете (до 90000Па - примерно до 600м)
 */
 
#define MinP   40000        //сдвиг давления при хранении в int (пример 101КПа -> 28000)
#define _cs    5 //3 //5  //0   CS         out
#define _mosi  6 //4 //6  //1   MOSI (SDI) out
#define _miso  4 //2 //4  //2   MISO (SDO) input
#define _sck   7 //5 //7        SCK        out

#define _gnd   8 //8   //GND BMP280
#define _vcc   9 //9   //VCC BMP280
#define led    13 //11  //светодиод тут
#define LED_PIN  led

#define test_in   10   //перемычка для сброса блокировки (вход)
#define test_out  11   //перемычка для сброса блокировки (выход)

#define _rf433    1    //передатчик RF433 подключаем к Tx
//#define _powOn    2    //включение транзистора питания (0 - вкл) подключаем к Tx в v.9 не использ.

#define _v_plus    A2  //10 //12  //A3  //динамик +
#define _v_minus   A2  //динамик -

#define _sw     2      //контакт датчика вибрации (второй на А5 
#define _sw2    A5     //контакт датчика вибрации 
#define _swu    10     //контакт второго датчика вибрации  
#define _swu2   A0     //контакт датчика вибрации 


#define _triak  A1 //2 //11 //10  //затвор MOSFET запала (1 - включить)
#define _reset  0   //при 0 на входе - принудительный сброс признаков старта, полета и парашюта

#define Vignit     A1 //A2      //анод тиристора (проверка обрыва игнитора
#define minVignit  400     //мин напр на аноде тиристора при исправном игниторе
#define Vacb       A1      //напряжение АКБ

//ячейки EEPROM
//#define a_fl_fly    3   //адрес ячейки с флагом "Полет был ли"
//#define a_fl_para   2   //адрес ячейки с флагом "Сигнал выброса парашюта был ли"
#define a_fl_start  1   //адрес ячейки с флагом "Обнаружен ли старт"

#include <EEPROM.h>

//распределение памяти EEPROM
#define a_fl_para       0 //byte признак выброса парашюта 1-было 
#define a_fl_fly        1 //byte признак состоявшегося полета (1-был,2-был, но с провалом питания) 
#define a_adr           2 //int адрес включения игнитора
#define a_vcc_mc_st     4 //int питание контроллера на старте
#define a_vcc_acc_st    6 //int питание АКБ
#define a_temp_st       8 //int температура
#define a_press_st     10 //int давление
#define a_vcc_mc_fin   12 //int питание контроллера в конце
#define a_vcc_acc_fin  14 //int питание АКБ
#define a_temp_fin     16 //int температура
//------- добавлено ускорение ---------
#define a_ax_st        18 //ускорение по Х на старте
#define a_ay_st        20 //ускорение по Y на старте
#define a_az_st        22 //ускорение по Z на старте
#define a_ax_fin       24 //ускорение по Х на финише (по оконч.регистр)
#define a_ay_fin       26 //ускорение по Y на финише (по оконч.регистр)
#define a_az_fin       28 //ускорение по Z на финише (по оконч.регистр)

#define N0 20            //начало области EEPROM для отсчетов высоты (24c256 - сначала)
#define Alt0  1025.25   //давление на высоте 0
#define MasMax 20   //10//20       //размер массива в ОЗУ (по 2*4 байта) для записи отсчетов до старта                        !!! 20 в 480
unsigned int addr = N0;     //пишем отсчеты не с начала EEPROM
unsigned int address = 1024*16;    // текущий адрес начала записи строки 32байтных отсчетов в EEPROM 24cXX
#define tPress 50  //100      //200      //период опроса датчика    (в 480 уменьшен до 50мс - на 25 сек                                         !!! 100
#define maxAddr 1024 //пишем в мегу    //1024*32 //512      //объем EEPROM ATtiny85 =512  //32К для 24С256  с 455                               !!! 1024
#define maxVCC  4000    //пороговое напряжение питания (мВ)
unsigned int maxAlt = 0;           //значение максимальной высоты (давление)
#define maxN 5          //число подряд идущих измерений высот меньших макс для опр.снижения
#define minD 50         //12 //60    //минимальное снижение от максимума для опр.снижения (на более чем 5м: 1м=12отсчетов)  !!!
int N = 0;              //подсчет числа повторов снижения высоты (до maxN)
int ignitON = 0;        //признак "включен ли запал" (0 - не включен)
byte aLow;
byte aHigh;
unsigned int valAlt;
long int valPress;
long int tTek;
//float tH0;
long int tBlokCCP;      //тут храним порог времени блокировки обнаружения падения
#define cBlokCCP 2500 //3500   //константа - на это время от старта блокируется срабатывание ССР (нач 2500)
long int tOnCCP;        //тут храним порог времени, когда принудительно включим ССР после старта
#define cOnCCP 15000  //20000    //константа - через это время после старта принудительно сработает ССР (нач 15000)

//long voltage;
//-------------
int Nbuf = 0;            //указатель ОЗУ (до старта отсчеты пишем в ОЗУ)
byte Mas[MasMax*2*4];    //сам массив в ОЗУ для хранения отсчетов до взлета  (10 ячеек по 8 байт)
unsigned int Hmax = 0;   //тут отслеживаем высоту (давление) при определении взлета 
unsigned int H0 = 0;     //тут высота (давление) на старте 
int Kpovt = 0;           //число повторов увеличения высоты при взлете
#define Hd 40 //18  //25 //60       //константа для определения взлета - падение давления (примерно 3м) 12 отсчетов на 1м (5м)  !!!
#define Kmax 3 //5       //константа число подряд изм.высоты больше макс. для опр.взлета

//--------------- блок GPS GSM ----------------
#define TimeSMS    15000      // периодичность автоотправки СМС (мс)
#define TimeSat    3500       // время ожидания спутников перед отпр.СМС
const String MY_NUMBER = "79913190757";
#define M590_BAUD      9600
//#include <AltSoftSerial.h>
// AltSoftSerial always uses these pins:
//
// Board          Transmit  Receive   PWM Unusable
// -----          --------  -------   ------------
// Arduino Uno        9         8         10
//AltSoftSerial m590Serial;
const String MY_NUMBER_WITH_PLUS = "+" + MY_NUMBER;
const String link = "http://maps.google.com/?q="; // <lat>,<lng>

unsigned int valSost;         //состояние системы
long int var_TimeSMS = 0;     // тут ведем отсчет времени между
                              // автоотправками СМС
boolean AutoSMS = false;      // признак автопосылки СМС
float NumSMS = 0;             // порядковый номер СМС (float c 309a)
bool typeSMS = true;          // пустая(true), с координатами(false)
String cmd;

//----- GPS
#define GPS_BAUD       9600
int fix;                      //чтоб не ругалось без GPS (заглушка старой версии)
byte chGpsLint[128];                                  // Переменная для хранения GPS-строки, char
word btReaded;                                        // её длина.
#define gpsBuffer chGpsLint   //чтоб не переписывать
bool bWaitingGps = true;                              // Флаг ожидания актуальных данных GPS.
char time1[16];               //временный буфер парсинга NMEA
unsigned int _tmpW = 0;                               // временная двухбайтовая переменная для преобразований


// --- eeprom
//#include <Wire.h>
//#include <EEEPROM.h>
//#include "I2Cdev.h"
//#include <ADXL345.h>
//ADXL345 accelerometer;
//#include "MPU6050.h"
//const int MPU_addr = 0x68; // I2C address of the MPU-6050
//MPU6050 accelgyro;
byte inputBytes[32] = { 0 }; //буфер данных для записи в 24C256
#define EEPROM_ADDRESS  0x50
//EEEPROM ee24c;
#define maxAddr24 maxAddr+32*512 // ограничен половиной от 1024 полный объем для 24с256  
#define sizestr 32       // размер записываемой за раз страницы в 24сХХ 
//Vector raw;              // данные с ахелерометра
unsigned int Atmp;
int addr_ind;
float AX;
float AY;
float AZ;
float Axyz;
int xVal, yVal, zVal;    //Variables for the values from the sensor MPU6050 accel


 /* -----------------------------------------------------------
 процедура "остановки" с заданной индикацией
 0 - двойные вспышки и длинные бипы
 1 - LED не меняется, *-*- пауза
 */
 void stopProc(int func = 0) {  
    if (func == 0) {   
 // останавливаемся с двойными вспышками и бипами
      while (1) {
        digitalWrite(led, HIGH);
        delay(200);               
        digitalWrite(led, LOW); 
        delay(200);               
        digitalWrite(led, HIGH);
        delay(200);               
        digitalWrite(led, LOW); 
        Beep();                   // звуковой сигнал 0,4 сек
        delay(600);               
        }
     }
    if (func == 1) {
      while (1) {
             Beep60();        
             delay(200);  
             Beep();                   // звуковой сигнал сброса флага запрета .-.-
             delay(200);               // короткий - длинный - короткий - длинный 
             Beep60();        
             delay(200);  
             Beep();        
             delay(1000); 
      } 
    } 
    if (func == 2) {
 	  //включим радиопередатчик с двойными "писками" и двойными длинный звуковой
//      pinMode(_rf433, OUTPUT);     
      while (1) {
//        tone200ms();               
        Beep();                   // звуковой сигнал 0,4 сек  ПискА
//        delay(200);               
//        tone200ms();               
        Beep();                   // звуковой сигнал 0,4 сек  ПискА
        delay(1000);               
        }
      }
      /*
    if (func == 3) {          //обрыв игнитора
          while(1) {
             Beep60();        //даем звуковое оповещение 
             delay(440);      //короткий 
             Beep();          //длинный
             delay(800); 
          }
    } 
    */
    if (func == 4) {          //ошибка инициализации датчика ВМР280
    while (1) {
        digitalWrite(led, HIGH);
        delay(200);               
        digitalWrite(led, LOW); 
        delay(200);               
        digitalWrite(led, HIGH);
        delay(200);               
        digitalWrite(led, LOW); 
        delay(200);               
        digitalWrite(led, HIGH);
        delay(200);               
        digitalWrite(led, LOW); 
        Beep();                   // звуковой сигнал ошибки ВМР -.-
        delay(200);               // длинный - короткий - длинный ПискВ
        Beep60();        
        delay(200);  
        Beep();        
        }
    }
    if (func == 5) {          //ошибка определения давления датчика ВМР280 (за пределами 0+80000..28000+80000)
    while (1) {
        digitalWrite(led, HIGH);  //коротк-длинная вспышка
        delay(200);               
        digitalWrite(led, LOW); 
        delay(200);               
        digitalWrite(led, HIGH);
        delay(800);               
        digitalWrite(led, LOW); 
        Beep();                   // звуковой сигнал ошибки ВМР -..
        delay(200);               // длинный - короткий - короткий ПискС
        Beep60();        
        delay(200);  
        Beep60();        
        }
    }
    if (func == 6) {          //останов с признаком "запись в EEPROM завершена"
      while (1) {
//        tone200ms();               
        digitalWrite(led, HIGH);
        Beep();                   // звуковой сигнал 0,4 сек
        digitalWrite(led, LOW);
        delay(800);               
        }
    }
    if (func == 7) {          //останов с признаком "запись в EEPROM завершена, но был провал питания" 
//       pinMode(_rf433, OUTPUT);     
     while (1) {
//        tone200ms();               
        delay(200);               
        Beep();                   // звуковой сигнал 0,4 сек
//        tone200ms();               
        delay(200);               
        Beep();                   // звуковой сигнал 0,4 сек
//        tone200ms();               
        delay(200);               
        Beep();                   // звуковой сигнал 0,4 сек
        delay(800);               
        }
    }
	//сигнал "по умолчанию"  
     while (1) {
        delay(200);               
        Beep();                   // звуковой сигнал 0,4 сек
        delay(200);               
        Beep();                   // звуковой сигнал 0,4 сек
        delay(200);               
        Beep();                   // звуковой сигнал 0,4 сек
        delay(200);               
        Beep();                   // звуковой сигнал 0,4 сек
        delay(800);               
        }
 }
 
 /* -----------------------------------------------------------
 процедура сброса признаков полета ракеты
 признак старта, признак выброса парашюта, признак прошедшего полета
 */
 void resetFlags(byte fl_start=0, byte fl_para=0, byte fl_fly=0) {     
 //    EEPROM.write(a_fl_start, fl_start);  //ставим признак - не было старта 
     EEPROM.write(a_fl_para, fl_para);    //ставим признак - не было выброса парашюта 
     EEPROM.write(a_fl_fly, fl_fly);      //ставим признак - не было полета 
 }


// процедура считывания данных с акселерометра
void ReadAxis() { 
  
    // read raw accel/gyro measurements from device
//    accelgyro.getMotion6(&ax, &ay, &az, &gx, &gy, &gz);
//    accelgyro.getMotion6(&xVal, &yVal, &zVal, &xgVal, &ygVal, &zgVal);

    // these methods (and a few others) are also available
//==========>    accelgyro.getAcceleration(&xVal, &yVal, &zVal);
    //accelgyro.getRotation(&gx, &gy, &gz);

}

void setup() {
  // initialize the digital pin as an output.
      pinMode(led, OUTPUT);     
      pinMode(_gnd, OUTPUT);   //общ BMP280  
      digitalWrite(_gnd, LOW); //общий провод ВМР280 на "землю"
      pinMode(_vcc, OUTPUT);   //питание BMP280  
      digitalWrite(_vcc, LOW); //питание ВМР280 на "землю" (выключим ВМР)
      pinMode(_triak, OUTPUT);     
      digitalWrite(_triak, LOW); //транзистор выключим
      pinMode(_v_plus, OUTPUT);     
      pinMode(_sw2, OUTPUT);     
      digitalWrite(_sw2, LOW);        // на "землю" второй контакт SW
      digitalWrite(_v_plus, LOW);     //"+" динамика на "землю"
      pinMode(_sw,INPUT);             //контакт SW - вход
      digitalWrite(_sw, HIGH);        //вкл подтягивающий резистор

      pinMode(_swu2, OUTPUT);     
      digitalWrite(_swu2, LOW);        // на "землю" второй контакт второго SW
      pinMode(_swu,INPUT);             //контакт второго SW - вход
      digitalWrite(_swu, HIGH);        //вкл подтягивающий резистор
      

    // Initialize EEPROM library.
//    Wire.begin(); // initialise the connection

      Beep60();     
      delay(400);  
//проверим напряжение питания
      valAlt = readVcc();                //питание устройства VCC(5V)    006
      if (valAlt > maxVCC) {
        //питаемся от 5В - включать BMP280 нельзя

        Beep();        //два бипа перед выводом
        delay(400);  
        Beep60();     
        
 //при питании 5В - включаем вывод данных по СОМ  ----------------------------------------
     Serial.begin(9600);  //v480
//        Serial.begin(9600);
//        Serial.println("Read EEPROM from MC & BMP280. Ver Pro_Bmp_TA_3V_v400a");
        Serial.println("v481");
        Serial.println(MinP);
        Serial.println("Adr;Low;(Sw)High;Word;Sw;SwU");
    
        for (addr = 0; addr < maxAddr; addr = addr + 2) {
            if (addr == N0) {
                   Serial.println("-------------;");    
                 }
           aLow  = EEPROM.read(addr);
           aHigh = EEPROM.read(addr+1);
           Serial.print(addr);
           Serial.print(";  ");
                   Serial.print(aLow);    //флаг парашюта для 0
                   Serial.print(";");
                   Serial.print(aHigh);  //флаг полета для 1
           Serial.print(";  ");
           AX = (float) (aLow + (aHigh | 0b11000000)*256.0) * 1.0;
           Serial.print(AX,0); 
           Serial.print(";  ");
           if ((aHigh & 0b10000000) == 0) Serial.print("0;  ");  //первый датчик
           else Serial.print("1;  ");     
           if ((aHigh & 0b01000000) == 0) Serial.print("0");  //второй датчик
           else Serial.print("1");     
//распределение памяти EEPROM
            if (addr == a_fl_para) {
                  //0 byte признак выброса парашюта 1-было 
                  //1 byte признак состоявшегося полета (1-был,2-был, но с провалом питания) 
                   Serial.print("; para; fly");    
                 }
            if (addr == a_adr) {
                  ////int адрес включения игнитора
                  Serial.print("; adr ignit");    
                 }
            if (addr == a_vcc_mc_st) {
                  //int питание контроллера на старте
                  Serial.print("; vcc_mc_st");    
                 }
            if (addr == a_temp_st) {
                  //int температура на старте
                  Serial.print("; temp_st");    
                 }
            if (addr == a_press_st) {
                  //int давление на старте
                  Serial.print("; press_st");    
                 }
            if (addr == a_vcc_mc_fin) {
                  //int питание контроллера в конце
                  Serial.print("; vcc_mc_fin");    
                 }
            if (addr == a_temp_fin) {
                  //int температура в конце
                  Serial.print("; temp_st");    
                 }
           Serial.println(";");
        }
//        Serial.println("The End Read EEPROM from ATMega328");
/*        
        Serial.println();  //"24C256: P AX AY AZ");
        for (addr = 0; addr < maxAddr; addr = addr + 8) {
          for (addr_ind = 0; addr_ind < sizestr; addr_ind++) {
              inputBytes[addr_ind] = ee24c.i2c_eeprom_read_byte(EEPROM_ADDRESS, addr + addr_ind);
          }
          Serial.print(addr);
          Serial.print(";");

          aLow  = inputBytes[0];
          aHigh = inputBytes[1];
          Serial.print(aLow+aHigh*256);  // давлениe
          Serial.print(";");
          
           Atmp = inputBytes[2]+inputBytes[3]*256;  //целое со знаком 2 байта 16000 = 16g
           AX = (float) Atmp/256.0;
           Serial.print(AX,4);      //ускорение по Х
           Serial.print(";");
           Atmp = inputBytes[4]+inputBytes[5]*256;  //целое со знаком 2 байта 16000 = 16g
           AY = (float) Atmp/256.0;
           Serial.print(AY,4);      //ускорение по Y
           Serial.print(";");
           Atmp = inputBytes[6]+inputBytes[7]*256;  //целое со знаком 2 байта 16000 = 16g
           AZ = (float) Atmp/256.0;
           Serial.print(AZ,4);      //ускорение по Z
           Serial.print(";");
           Axyz = sqrt(AX*AX + AY*AY + AZ*AZ);
           Serial.print(Axyz,4);    //общее ускорение
           Serial.println(";");
  */
         /*  
          Atmp = int(inputBytes[2]+inputBytes[3]*256);  //целое со знаком 2 байта
          Serial.print(Atmp);  //ускорение Х
          Serial.print(";");
          Atmp = int(inputBytes[4]+inputBytes[5]*256);  //целое со знаком 2 байта
          Serial.print(Atmp);  //ускорение Y
          Serial.print(";");
          Atmp = int(inputBytes[6]+inputBytes[7]*256);  //целое со знаком 2 байта
          Serial.print(Atmp);  //ускорение Z
          Serial.println(";");
          */
    //    }
        
        Serial.println();   //"24C256 next:");
        /*
        for (addr = maxAddr; addr < maxAddr24; addr = addr + sizestr) {    //maxAddr24
          for (addr_ind = 0; addr_ind < sizestr; addr_ind++) {
              inputBytes[addr_ind] = ee24c.i2c_eeprom_read_byte(EEPROM_ADDRESS, addr + addr_ind);
          }
          Serial.print(addr);
          Serial.print(";NA;");
          aLow  = inputBytes[0];
          aHigh = inputBytes[1];
          Serial.print(aLow+aHigh*256);  //не исп
          Serial.print(";A;");
          aLow  = inputBytes[2];
          aHigh = inputBytes[3];
          Serial.print(aLow+aHigh*256);  //адрес соотв отсчета в АТmega
          Serial.print(";T;");
          aLow  = inputBytes[4];
          aHigh = inputBytes[5];
          Serial.print(aLow+aHigh*256);  //часы мин GPS
          Serial.print(";");
          aLow  = inputBytes[6];
          aHigh = inputBytes[7];
          Serial.print(aLow+aHigh*256);  //сек десят GPS
          
          Serial.print(";LAT;");
          aLow  = inputBytes[8];
          aHigh = inputBytes[9];
          Serial.print(aLow+aHigh*256);  //latitude deg,min
          Serial.print(";");
          aLow  = inputBytes[10];
          aHigh = inputBytes[11];
          Serial.print(aLow+aHigh*256);  //lat sec,dec

          Serial.print(";LNG;");
          aLow  = inputBytes[12];
          aHigh = inputBytes[13];
          Serial.print(aLow+aHigh*256);  //longtitude deg,min
          Serial.print(";");
          aLow  = inputBytes[14];
          aHigh = inputBytes[15];
          Serial.print(aLow+aHigh*256);  //lng sec,dec

          Serial.print(";S;");
          aLow  = inputBytes[16];
          aHigh = inputBytes[17];
          Serial.print(aLow+aHigh*256);  //спутники
          Serial.print(";ALT;");
          aLow  = inputBytes[18];
          aHigh = inputBytes[19];
          Serial.print(aLow+aHigh*256);  //altitude

          Serial.print(";V;");
          aLow  = inputBytes[20];
          aHigh = inputBytes[21];
          Serial.print(aLow+aHigh*256);  //напряжение
          Serial.print(";P;");
          aLow  = inputBytes[22];
          aHigh = inputBytes[23];
          Serial.print(aLow+aHigh*256);  // давлениe
          
          Serial.print(";C;");
          aLow  = inputBytes[24];
          aHigh = inputBytes[25];
          Serial.print(aLow+aHigh*256);  //температура
          
          Serial.print(";X;");
          Atmp = inputBytes[26]+inputBytes[27]*256;  //целое со знаком 2 байта
           AX = (float) Atmp/256.0;
           Serial.print(AX,4);      //ускорение по Х
          Serial.print(";Y;");
          Atmp = inputBytes[28]+inputBytes[29]*256;  //целое со знаком 2 байта
           AY = (float) Atmp/256.0;
           Serial.print(AY,4);      //ускорение по Y
          Serial.print(Atmp);  //ускорение Y
          Serial.print(";Z;");
          Atmp = inputBytes[30]+inputBytes[31]*256;  //целое со знаком 2 байта
           AZ = (float) Atmp/256.0;
           Serial.print(AZ,4);      //ускорение по Z
          Serial.print(Atmp);  //ускорение Z
          Serial.print(";");
           Axyz = sqrt(AX*AX + AY*AY + AZ*AZ);
           Serial.print(Axyz,4);    //общее ускорение
           Serial.println(";");
        }
        */
//     Serial.println("The End Read EEPROM");
        /*
        Serial.println(); // "for MFR.EXE");
          //в шапке укажем начальное время старта из 24сХХ 
          for (int addr_ind = 0; addr_ind < sizestr; addr_ind++) {
              inputBytes[addr_ind] = ee24c.i2c_eeprom_read_byte(EEPROM_ADDRESS, 0+16*1024);
          }
          */
        Serial.println("'v481");
//          Serial.println("'N;T;P;*C;X-;Y-;SW;SWU;");
          /*
          aLow  = inputBytes[4];
          aHigh = inputBytes[5];
          Serial.print(aHigh);      //час
          Serial.print("h ");
          Serial.print(aLow);       //мин
          Serial.print("m ");
          aLow  = inputBytes[6];
          aHigh = inputBytes[7];
          Serial.print(aHigh);      //сек  GPS
          Serial.println("s  ");
          */
          //нужна температура, берем с точки старта
          addr = a_temp_st;
          aLow  = EEPROM.read(addr);
          aHigh = EEPROM.read(addr+1);
          float tempD = (float) (aLow+aHigh*256.0 - 2000.0)/100.0;
/*
#define a_adr           2 //int адрес включения игнитора
#define a_vcc_mc_st     4 //int питание контроллера на старте
#define a_vcc_acc_st    6 //int питание АКБ
#define a_temp_st       8 //int температура
#define a_press_st     10 //int давление
*/
        float timeD = 0;
        int numD = 0;
        for (addr = N0; addr < maxAddr; addr = addr + 2) {
           aLow  = EEPROM.read(addr);
           aHigh = EEPROM.read(addr+1);
           Serial.print(numD);    //порядковый номер записи (с 0)
           Serial.print(";");
           Serial.print(timeD);   //время в сек
           Serial.print(";");
           AX = (float) (MinP + aLow + (aHigh | 0b11000000)*256.0) * 1.0;
//           AX = (float) (MinP + aLow + aHigh*256.0) * 1.0;
           Serial.print(AX,0);  //высота/давление
           Serial.print(";");
          //нужна температура, берем с яч.[2]&[3]
//           tempD = (float) (inputBytes[2]+inputBytes[3]*256 - 2000)/100.0;
           Serial.print(tempD,2);   //температура в *С
           Serial.print(";");
           /*
//           Atmp = inputBytes[2]+inputBytes[3]*256;  //целое со знаком 2 байта 16000 = 16g
//           AX = (float) Atmp/256.0;
//           Serial.print(AX,4);      //ускорение по Х
           Serial.print("0;");
//           Atmp = inputBytes[4]+inputBytes[5]*256;  //целое со знаком 2 байта 16000 = 16g
//           AY = (float) Atmp/256.0;
//           Serial.print(AY,4);      //ускорение по Y
           Serial.print("0;");
//           Atmp = inputBytes[6]+inputBytes[7]*256;  //целое со знаком 2 байта 16000 = 16g
//           AZ = (float) Atmp/256.0;
//           Serial.print(AZ,4);      //ускорение по Z
           Serial.print("0;");
           //вместо общего ускорения:
//           Axyz = sqrt(AX*AX + AY*AY + AZ*AZ);
//           Serial.print(Axyz,4);    //общее ускорение
           //выводим напряжение на контроллере в В
//           Atmp = inputBytes[6]+inputBytes[7]*256;  //целое со знаком 2 байта (миливольты)
//           AZ = (float) Atmp/1000.0;
//           Serial.print(AZ,3);      //напряжение питания
           Serial.println(";");
           */
           Serial.print("0;1;");  //ускорение не фиксируем
           if ((aHigh & 0b10000000) == 0) Serial.print("0;");  //первый датчик на месте Z
           else Serial.print("1;");     
           if ((aHigh & 0b01000000) == 0) Serial.println("0;");  //второй датчик
           else Serial.println("1;");     
 //         Serial.println("0;1;0;1;");  //ускорение не фиксируем

           numD ++;
           timeD += 0.05;   //дискрет времени в 480 50мс
        }
        Serial.println("'===END OF RECORD===");

        resetFlags(0,0,0);            //сбросить все признаки
        stopProc(0);
      }
   //---------------------- окончание вывода (+5В) --------------------------
/*
   //имеем 3В питание, но могут быть замкнуты Тх-Rx для сброса признаков
      digitalWrite(_reset, HIGH); //подтянем вход Rx на VCC
      pinMode(_reset, INPUT);    
      pinMode(_rf433, OUTPUT);    //Tx как выход
      digitalWrite(_rf433, LOW);  //0
      if (digitalRead(_reset) == LOW)  {  // OK - можно сбросить ячейку
             resetFlags(0,0,0);
             digitalWrite(led, HIGH);      //вкл Led - все ОК
             stopProc(1);              // останов             
		  }
     */
//--------- окончание анализа сигналов сброса признаков ------------

/*
//проверка флага "был ли сигнал выброса парашюта" 
      aLow  = EEPROM.read(a_fl_para);  
      if (aLow == 1) {
        //тут надо продолжить регистрацию до заполнения памяти
        pinMode(_vcc, OUTPUT);     
        digitalWrite(_vcc, HIGH); //питание ВМР280 на VCC
        delay(100);               
 
        if (!BMP280_begin()) {  //если ошибка BMP280 - тройные вспышки
             stopProc(8);       // останов с признаком "ошибка ВМР280 после парашюта"            
            }
        //ОК - BMP280 работает
		valAlt = readPressure();  //потренируем
        delay(100);               
		valAlt = readPressure();  //потренируем
		
		//восстановим адрес записи в регистратор из EEPROM (мы его запомнили перед включением запала)
           byte aLow  = EEPROM.read(a_adr);
           byte aHigh = EEPROM.read(a_adr+1);
		   //addr = (aHigh<<8) | aLow;
		   addr = aHigh*256 + aLow;
		
		
while (1)	{	
  tTek = millis();
  
//Высота  (давление)
  valAlt = readPressure();
  EEPROM.write(addr, lowByte(valAlt));
  EEPROM.write(addr+1, highByte(valAlt));
//  MasToEE(valAlt);  //запись в EEPROM  !!!!!! там в ОЗУ идет перезапись - надо убрать!
  addr = addr + 2;
  if (addr >= maxAddr)  {
   // запись завершена - фиксируем конечные данные
     valAlt = BMP280_readTemperature() +2000; //температура с BMP280 на 20 градусов выше и в 100 раз больше
     EEPROM.write(a_temp_fin, lowByte(valAlt));
     EEPROM.write(a_temp_fin+1, highByte(valAlt));
     valAlt = readVcc();                //питание устройства VCC(5V)    002
     EEPROM.write(a_vcc_mc_fin, lowByte(valAlt));
     EEPROM.write(a_vcc_mc_fin+1, highByte(valAlt));

     voltage = analogRead(Vacb) * 2;      //напр. аккумулятора (там делитель  на 2)
     voltage = (voltage * valAlt) / 1024;        //пересчет в мВ с учетом питания МК
     H0 = voltage;
     EEPROM.write(a_vcc_acc_fin, lowByte(H0));
     EEPROM.write(a_vcc_acc_fin+1, highByte(H0));

     //память заполнена - светодиод  ГАСИМ
     digitalWrite(led, LOW);
     //отключаем выводы BMP280
     pinMode(_cs, INPUT);
     pinMode(_sck, INPUT);
     pinMode(_mosi, INPUT);
     pinMode(_miso, INPUT);
     digitalWrite(_vcc, LOW); //снимем питание
     pinMode(_vcc, INPUT);
     pinMode(_gnd, INPUT);

     resetFlags(1,1,2);            //установить все признаки, в т.ч. был полет после провала питания(2)
     //включаем передатчик (одиночные писки)       
      pinMode(_rf433, OUTPUT);     
      stopProc(7);              // останов с признаком "запись в EEPROM завершена, но был провал питания"            
  }
    
  while ((tTek + tPress) > millis()) {} 
} 
} 
*/      
//------------------------- собственно начало программы -------------------------------

/*  убрано в версии 12сА
//первым делом проверим "запал" на обрыв
  valAlt = analogRead(Vignit);      //на стоке (игниторе)
  if (valAlt < minVignit) {
        //на стоке нет напряжения - обрыв игнитора
             stopProc(3);              // останов с признаком "обрыв игнитора"            
    }   
*/
      addr = N0;      //пишем отсчеты не с начала EEPROM
//  itmp = 0;
//  maxAlt = 0;     //значение максимальной высоты
//  N = 0;    //подсчет числа повторов снижения высоты (до maxN)
//  ignitON = 0;    //признак "включен ли запал" (0 - не включен)
      Nbuf = 0;       //указатель ОЗУ (до старта отсчеты пишем в ОЗУ)
      Hmax = 0;       //тут отслеживаем высоту при определении взлета 
      H0 = 0;         //тут высота на старте 
//  Kpovt = 0;      //число повторов увеличения высоты при взлете
  

      Beep();        //один длинный бип после включения
      delay(1000);               
      
//      pinMode(_rf433, OUTPUT);     
      for (int i = 0; i < 5; i ++) {  //и 5 писков в радиоканал
//        tone200ms();                  //для контроля ПРД
        Beep60();
        delay(400);               
        }
        
		// разрешается фиксировать параметры датчика в EEPROM
/*
      //---- настраиваем и проверяем акселерометр
      accelgyro.initialize();  //в библиотеке переделано на 16g
      // verify connection
      if (accelgyro.testConnection()) {} //"MPU6050 connection successful"
      else {  // "MPU6050 connection failed");
        //что нибудь сделать для индикации
              digitalWrite(led, HIGH);      //вкл Led - ошибка
      
             Beep60();        
             delay(200);  
             Beep();                   // звуковой сигнал ошибки 6050 .-.-
             delay(200);               // короткий - длинный - короткий - длинный 
             Beep60();        
             delay(200);  
             Beep();        
             delay(1000);               
     }
*/
      pinMode(_vcc, OUTPUT);     
      digitalWrite(_vcc, HIGH); //питание ВМР280 на VCC
      //пауза 2 сек
      for (int i = 0; i < 10; i ++) {
        digitalWrite(led, HIGH);
        delay(100);               
        digitalWrite(led, LOW); 
        delay(100);               
        }
//        Beep();                   // звуковой сигнал 0,4 сек
 
  if (!BMP280_begin()) {  //если ошибка BMP280 - тройные вспышки
             stopProc(4);              // останов с признаком "ошибка ВМР280"            
    }

//  if (!accelerometer.begin())
//  {
//             stopProc(4);              // останов с признаком "ошибка ADXL345"  (no 4)            
//    Serial.println("Could not find a valid ADXL345 sensor, check wiring!");
//    delay(500);
//  }

  // Set measurement range
  // +/-  2G: ADXL345_RANGE_2G
  // +/-  4G: ADXL345_RANGE_4G
  // +/-  8G: ADXL345_RANGE_8G
  // +/- 16G: ADXL345_RANGE_16G
//  accelerometer.setRange(ADXL345_RANGE_16G);
//    accelgyro.setFullScaleAccelRange(3); //ставим 16g 
    //хотя в библиотеке 16g уже прописано
//    inputBytes[6] = accelgyro.getFullScaleAccelRange();  //для проверки глянем диапазон 6050 (0=2g,3=16g)

//  m590Serial.begin(M590_BAUD);
//  Serial.begin(GPS_BAUD);       //тут Neo-6M GPS
//  delay(300);                   // wait 
  
  //------------------------------------------------------------
  // M590 GSM modem config
//  digitalWrite(M590_DTR, HIGH);

  digitalWrite(LED_PIN, HIGH);
  newDelay(250);
  digitalWrite(LED_PIN, LOW);
  
  // Basic health check
  /*
  sendAndWait("AT", "OK");
  cmd = "ATE0";   //выкл эхо
  m590Serial.println(cmd);

  digitalWrite(LED_PIN, HIGH);
  newDelay(250);
  digitalWrite(LED_PIN, LOW);
  newDelay(250);
  digitalWrite(LED_PIN, HIGH);
  newDelay(500);
  digitalWrite(LED_PIN, LOW);
*/
/* в 309 нет анализа вызова
  // Get caller's number  Включение режима отображения номера звонящего
  // в строке RING будет присутствовать номер звонящего абонента
  // по умолчанию (AT+CLIP=0) режим отключен - идет просто "RING"
  sendAndWait("AT+CLIP=1", "OK");

  digitalWrite(LED_PIN, HIGH);
  newDelay(250);
  digitalWrite(LED_PIN, LOW);
  newDelay(250);
  digitalWrite(LED_PIN, HIGH);
  newDelay(250);
  digitalWrite(LED_PIN, LOW);
  newDelay(250);
  digitalWrite(LED_PIN, HIGH);
  newDelay(500);
  digitalWrite(LED_PIN, LOW);
*/  

  // Set SMS mode to text
  //sendAndWait("AT+CMGF=1", "OK");

  // Set character set to GSM (aka ASCII)
  //sendAndWait("AT+CSCS=\"GSM\"", "OK");
  // End of M590 GSM modem config
  //------------------------------------------------------------

  digitalWrite(LED_PIN, HIGH);
  newDelay(250);
  digitalWrite(LED_PIN, LOW);
  newDelay(250);
  digitalWrite(LED_PIN, HIGH);
  newDelay(250);
  digitalWrite(LED_PIN, LOW);
  newDelay(250);
  digitalWrite(LED_PIN, HIGH);
  newDelay(250);
  digitalWrite(LED_PIN, LOW);
  newDelay(250);
  digitalWrite(LED_PIN, HIGH);
  newDelay(500);
  digitalWrite(LED_PIN, LOW);

  // Wait for SIM to be registered in local network
  //sendAndWait("AT+CREG?", "+CREG: 0,1");

  // One long blink - GSM ready
  //digitalWrite(LED_PIN, HIGH);
  newDelay(500);
  //digitalWrite(LED_PIN, LOW);
/*
  //------------------------------------------------------------
  GPSFlush(); //чистим буфер GPS и ждем его наполнения  
  // Wait until the fix is available
  while (getFix()) {
    for (int iWait = 0; iWait < 2; iWait ++) {
       digitalWrite(LED_PIN, HIGH);  // индик.ожид.коротк.3 вспышки
       Beep60();
       digitalWrite(LED_PIN, LOW);
       newDelay(100);
       }
       newDelay(140);
    }   // ждем спутники
*/

//проверка флага "был ли полет" 
      aLow  = EEPROM.read(a_fl_fly);  
      if (aLow > 0) {
                  Beep60();
                  delay(100);      
                  Beep60();
                  delay(100);      
                  Beep60();
//            stopProc(2);              // останов с признаком "сбой по питанию после полета"  
             //тут надо периодическую отправку СМС          

             for (int _sms=0; _sms < 8; _sms++) { //примерно 2 мин шлем СМС (15 сек)

                  newDelay(10000);  //пауза 10 сек
                  // треск 5 сек
                  for (int z = 0; z < 30; z++) {
                  Beep60();
                  delay(100);      
                       }
             // отправка  СМС
             digitalWrite(LED_PIN, HIGH);
/*
             // Get a fix and write it to global variable
             for (int stl = 1; stl < 5; stl++) {
                   if (!getFix()) break;  //ждем координаты
                   newDelay(200);  //пауза 
                  }
             valSost = 4000;  //состояние отправки СМС 
             AutoSMS = true;  //A - признак сбоя питания
             sendSms(0, MY_NUMBER_WITH_PLUS);
*/
             digitalWrite(LED_PIN, LOW);
             }
             stopProc(2);              // останов с признаком "сбой по питанию после полета"  
      }


  // Two short blinks - GPS ready
  digitalWrite(LED_PIN, HIGH);
  newDelay(200);
  digitalWrite(LED_PIN, LOW);
  newDelay(100);
  digitalWrite(LED_PIN, HIGH);
  newDelay(200);
  digitalWrite(LED_PIN, LOW);

// настройки завершены - треск
  newDelay(1000);
  for (int z = 0; z < 10; z++) {
      Beep60();
      delay(100);      
   }


// отправка контрольной (первой СМС) после включения

  
  digitalWrite(LED_PIN, HIGH);

  // Get a fix and write it to global variable
  // typeSMS = getFix();  //убрано, т.к. спутники пойманы

      Hmax = readPressure();  //на самом деле - "тренировка датчика перед реальным считыванием"
      delay(400);  
      Hmax = readPressure();  //это давление шлем в первой СМС (чтоб более-менее реально)

  valSost = 4000;  //состояние отправки СМС (первая)
  sendSms(Hmax, MY_NUMBER_WITH_PLUS);

  digitalWrite(LED_PIN, LOW);
//  m590Serial.setTimeout(400);


//ОК - BMP280 работает
      Beep60();        //четыре бипа после обнаружения нормального отклика BMP280
      Hmax = readPressure();  //на самом деле - "тренировка датчика перед реальным считыванием"
      delay(400);  
      Beep60();        
      Hmax = readPressure();  //на самом деле - "тренировка датчика перед реальным считыванием"
      delay(400);  
      Beep60();        
      Hmax = readPressure();  //на самом деле - "тренировка датчика перед реальным считыванием"
      delay(400);  
      Beep60();        

  //пауза перед началом обнаружения старта - три двойные вспышки
      delay(5000);  
  /*
  for (int itmp = 0; itmp < 3; itmp++) {  //30
     digitalWrite(led, HIGH);   // turn the LED on (HIGH is the voltage level)
     delay(200);
     digitalWrite(led, LOW);    // turn the LED off by making the voltage LOW
     delay(200);
     digitalWrite(led, HIGH);   // turn the LED on (HIGH is the voltage level)
     delay(200);
     digitalWrite(led, LOW);    // turn the LED off by making the voltage LOW
//     Beep();                    // звуковой сигнал 0,4 сек
      Hmax = readPressure();  //на самом деле - "тренировка датчика перед реальным считыванием"
     delay(800);
  }   
 */ 
   
//=========== пишем параметры на старте ===========
//еще раз "оживим датчик"

  valAlt = BMP280_readTemperature() +2000; //температура с BMP280    на 20 градусов выше и в 100 раз больше
//  EEPROM.write(0, lowByte(valAlt));
//  EEPROM.write(1, highByte(valAlt));
  valAlt = readVcc();                //питание устройства VCC(5V)    006
  EEPROM.write(a_vcc_mc_st, lowByte(valAlt));
  EEPROM.write(a_vcc_mc_st+1, highByte(valAlt));
/*
     voltage = analogRead(Vacb) * 2;      //напр. аккумулятора (там делитель  на 2)
     voltage = voltage * valAlt / 1024;        //пересчет в мВ с учетом питания МК
     H0 = voltage;
     EEPROM.write(a_vcc_acc_st, lowByte(H0));
     EEPROM.write(a_vcc_acc_st+1, highByte(H0));
*/  
  //определение исходного давления : три отсчета через 500 мс, берем среднее
//Hmax = BMP280_readTemperature();
Hmax = readPressure();  //на самом деле - давление
delay(500);
H0 = readPressure();  //на самом деле - давление
if (H0 >= Hmax) {
  valAlt = H0;
  H0 = Hmax;
  Hmax = valAlt;
}
// тут имеем H0 <= Hmax
delay(500);
valAlt = readPressure();  //на самом деле - давление
if (valAlt < Hmax) {
  if (valAlt <= H0) Hmax = H0;
  else Hmax = valAlt;
}
  H0 = Hmax;   //высота (давление) перед стартом
//  maxAlt = Hmax;  //тут храним минимум давления (макс высота)
/*     ===== проверка с v460 отключена =====
//  считаем нормальным давление в пределах 98000..105000
if ((Hmax <= 0) || (Hmax > 31000)) {
//неверное определение давления - ошибка датчика или сбой
      Beep60();        //даем звуковое оповещение 
      delay(400);      //короткий - длинный
      Beep();        
      delay(1000);  

//вторая попытка определения начального давления
Hmax = readPressure();  //на самом деле - давление
delay(500);
H0 = readPressure();  //на самом деле - давление
if (H0 >= Hmax) {
  valAlt = H0;
  H0 = Hmax;
  Hmax = valAlt;
}
// тут имеем H0 <= Hmax
delay(500);
valAlt = readPressure();  //на самом деле - давление
if (valAlt < Hmax) {
  if (valAlt <= H0) Hmax = H0;
  else Hmax = valAlt;
}
  H0 = Hmax;   //высота (давление) перед стартом
//  maxAlt = Hmax;  //тут храним минимум давления (макс высота)
if ((Hmax <= 0) || (Hmax > 31000)) {
//снова не верное определение давления - ошибка датчика или сбой  - встаем в цикле
             stopProc(5);              // останов с признаком "ошибка определения давления ВМР280"            
}
}
*/
//пишем начальное значение давления на старте   
  EEPROM.write(a_press_st, lowByte(H0));
  EEPROM.write(a_press_st+1, highByte(H0));

  maxAlt = Hmax;  //тут будем определять минимум давления (макс высота)


// отправка контрольной (первой СМС) после включения

  
//  digitalWrite(LED_PIN, HIGH);

  // Get a fix and write it to global variable
  // typeSMS = getFix();  //убрано, т.к. спутники пойманы

  valSost = 4000;  //состояние отправки СМС (первая)
//  sendSms(maxAlt, MY_NUMBER_WITH_PLUS);  //в 309b убрана - сильно влияет на давление - переставлено в начало

//  digitalWrite(LED_PIN, LOW);
//  m590Serial.setTimeout(400);

/*
  // читаем "сырые" данные с акселерометра (двухбайтовые)
     ReadAxis(); //ускорение
//  raw = accelerometer.readRaw();
  // запишем состояние "на старте"
  //Serial.print(raw.XAxis); 
//  Atmp = raw.XAxis;
  EEPROM.write(a_ax_st, lowByte(xVal));
  EEPROM.write(a_ax_st+1, highByte(xVal));
  //Serial.print(raw.YAxis);
//  Atmp = raw.YAxis;
  EEPROM.write(a_ay_st, lowByte(yVal));
  EEPROM.write(a_ay_st+1, highByte(yVal));
  //Serial.print(raw.ZAxis); 
//  Atmp = raw.ZAxis;
  EEPROM.write(a_az_st, lowByte(zVal));
  EEPROM.write(a_az_st+1, highByte(zVal));
*/
        valAlt  = maxAlt;  
        APVtoGpsLint();
        //спутники есть и данные GPS надо сохранить в 24с
//        ee24c.i2c_eeprom_write_page(EEPROM_ADDRESS, address, chGpsLint, sizestr);  //
        address += sizestr; // было sizestr;                      //сдвигаем адрес на пустое место
        // "под размер страницы" - чтоб не было ложной перезаписи данных

//отсчеты давления нормальные - даем контрольный сигнал (5 коротких писков)  
      Beep60();        //пять бип после фиксации нормального давления
      delay(400);  
      Beep60();        
      delay(400);  
      Beep60();        
      delay(400);  
      Beep60();        
      delay(400);  
      Beep60();        
//этот же сигнал - признак перехода к обнаружению старта
      delay(1000);  

//int ii = 0;
  //пишем отсчеты в ОЗУ до обнаружения старта
 do {
    long int yTek = millis();
    valAlt = readPressure();
//    if ((valAlt <= 0) || (valAlt > 31000)) {
    //неверное определение давления - ошибка датчика или сбой 
    //в память не пишем, в определении взлета не используем
//           Beep();                  // индикатор - динный писк
//   }
//    else 
    AltToMas(valAlt);  //пишем отсчет в ОЗУ

     //контрольное мигание вспышки 1/8   2 сек
     
      int yTmp = highByte(yTek) & B00000111;
      //было - контрольное мигание по три вспышки 1/8   2 сек
//     if ((yTmp == 0) || (yTmp == 2) || (yTmp == 4))  {   
      if (yTmp == 0)  {   
          digitalWrite(led, HIGH);   // turn the LED on (HIGH is the voltage level)
          Beep60();                  // короткий писк (реально 2-3 писка)
          }
      else {
          digitalWrite(led, LOW);    // turn the LED off by making the voltage LOW
          } 
    
    if (valAlt < Hmax) {    //до 308 было <=
      //с высотой давление падает, поэтому ищем понижение
       Hmax = valAlt;
       Kpovt ++;      
     }
    else {
       Kpovt = 0;  //сброс числа повторений
       Hmax = H0;  //сброс начальной высоты (добавлено в 306b)
     } 

   while ((yTek + tPress) > millis()) {}  //пауза перед след.отсчетом

} while(Hmax >= (H0 - Hd) || Kpovt < Kmax);   //пока давление не упало на Hd и число повторов падения давления меньше Кмакс

  //летим...  идем в основной цикл регистрации
  Nbuf = (MasMax-1)*2*4;           //подготовка к перезаписи ОЗУ в EEPROM
//  addr = MasMax*2*4 + N0;          //адрес EEPROM для фиксации высоты
  addr = MasMax*2 + N0;          //адрес EEPROM для фиксации высоты

  //гасим светодиод - признак "летим"
//          digitalWrite(led, LOW);    // turn the LED off by making the voltage LOW

  valAlt = BMP280_readTemperature() +2000; //температура с BMP280    на 20 градусов выше и в 100 раз больше
  EEPROM.write(a_temp_st, lowByte(valAlt));
  EEPROM.write(a_temp_st+1, highByte(valAlt));
  
  
				//выставим флаги полета
                //признак старта=1, признак выброса парашюта=0, признак прошедшего полета=1
				resetFlags(1,0,1);            //установить все признаки
//       a_fl_fly = 1  как бы полетели, регистрация пошла, будет сбой - регистрацию не вкл!
//------------------ добавка с версии v9 ----------------------
  tTek = millis();
  //первый отсчет полета пишем в край ОЗУ (убираем задвоение в v8) 
  Mas[Nbuf+2] = lowByte(valAlt);        //температура вместо Х (см выше)
  Mas[Nbuf+3] = highByte(valAlt);
  valAlt = readPressure();              //Высота  (давление)
  Mas[Nbuf] = lowByte(valAlt);
  //в старший байт в бит 7 ставим состояние датчика вибрации (0-замкнут)
  if (digitalRead(_sw) == LOW) Mas[Nbuf+1] = 0b01111111 & highByte(valAlt);
  else Mas[Nbuf+1] = 0b10000000 | highByte(valAlt);
  //в старший байт в бит 6 ставим состояние второго датчика вибрации (0-замкнут)
  if (digitalRead(_swu) == LOW) Mas[Nbuf+1] = 0b10111111 & Mas[Nbuf+1];
  else Mas[Nbuf+1] = 0b01000000 | Mas[Nbuf+1];
  valAlt = readVcc();                   //питание устройства VCC(5V)    006
  Mas[Nbuf+6] = lowByte(valAlt);        //вместо Z
  Mas[Nbuf+7] = highByte(valAlt);
  /*
  raw = accelerometer.readRaw();
  // запишем ускорение
  Atmp = raw.XAxis;
  Mas[Nbuf+2] = lowByte(Atmp);   //X
  Mas[Nbuf+3] = highByte(Atmp);
  Atmp = raw.YAxis;
  Mas[Nbuf+4] = lowByte(Atmp);   //Y
  Mas[Nbuf+5] = highByte(Atmp);
  Atmp = raw.ZAxis;
  Mas[Nbuf+6] = lowByte(Atmp);   //Z
  Mas[Nbuf+7] = highByte(Atmp);
  */
  while ((tTek + tPress) > millis()) {}

  // определим время блокировки алгоритма определения падения
  tBlokCCP = millis() + cBlokCCP;
  // определим время принудительного срабатывания ССР
  tOnCCP = millis() + cOnCCP;
   
}

void loop() {
  tTek = millis();
  
//Высота  (давление)
  valAlt = readPressure();
  MasToEE(valAlt);  //запись в EEPROM  
  

//**************************************************************************************
  //определяем момент включения "запала системы спасения"
  if (ignitON == 0) {  //только если запал еще не включали 
    //с 308 добавлено время принудительного включения ССР
    if (millis() > tOnCCP) {  
        //запишем в EEPROM момент включения "запала" (addr+2)
        EEPROM.write(a_adr, lowByte(addr+2));
        EEPROM.write(a_adr+1, highByte(addr+2));
        //выставим флаги полета
                //признак старта=1, признак выброса парашюта=1, признак прошедшего полета=1 (до 308 был 0)
        resetFlags(1,1,1);            //установить все признаки
        
        //включим "запал"
        digitalWrite(_triak, HIGH);
        ignitON = 1; //признак "Запал включен"
        delay(10); //подождем реакции
        digitalWrite(LED_PIN, HIGH);

        // Get a fix and write it to global variable
        typeSMS = getFix();

        //valAlt = maxAlt;            // max высота (миним.давление)
        valSost = 3000;  //состояние отправки СМС (вкл игнитор)
        AutoSMS = true;  //в СМС символ "A"
        sendSms(maxAlt, MY_NUMBER_WITH_PLUS);
        AutoSMS = false; //в СМС потом символ "S"

        digitalWrite(LED_PIN, LOW);
        var_TimeSMS = millis() + TimeSMS;  //время отправки следующей СМС после вкл игнитора
         }
  if (valAlt < maxAlt) {  //"копим" максимум высоты - идет "взлет" (падение давления)
      maxAlt = valAlt;
	  N = 0;              //сброс счетчика повтора отсчетов со снижением
	  }
  else {  //возможно, пошло падение (либо "зависли" на одной высоте :)
    //с 308 добавлено время блокировки - защита от возможных "подземлю" при резком старте
    if (millis() > tBlokCCP) {  
      N ++;  //считаем число повторов "понижения"
	  if (N > maxN) {  //число подряд "понижений" превысило порог - можно сказать - "падаем"
	      //проверим абсолютное значение "падения" к максимуму
	      if ((valAlt - maxAlt) > minD) { //тут пора включать "запал"

			  //запишем в EEPROM момент включения "запала" (addr+2)
			  //после включения игнитора может "провалиться" питание и контроллер перестанет работать
			  //после перегорания нити питание восстановится - регистрация должна продолжиться с этого адреса
				EEPROM.write(a_adr, lowByte(addr+2));
				EEPROM.write(a_adr+1, highByte(addr+2));
				//выставим флаги полета
                //признак старта=1, признак выброса парашюта=1, признак прошедшего полета=1 (до 308 был 0)
				resetFlags(1,1,1);            //установить все признаки
				
			  //включим "запал"
				digitalWrite(_triak, HIGH);
				ignitON = 1; //признак "Запал включен"
        delay(10); //подождем реакции
        digitalWrite(LED_PIN, HIGH);

        // Get a fix and write it to global variable
        typeSMS = getFix();

        //valAlt = maxAlt;            // max высота (миним.давление) в СМС 2S
        valSost = 3000;             //состояние отправки СМС (вкл игнитор)
        sendSms(maxAlt, MY_NUMBER_WITH_PLUS);

        digitalWrite(LED_PIN, LOW);
        var_TimeSMS = millis() + TimeSMS;  //время отправки следующей СМС после вкл игнитора
		     }
	     }
    } 
	  }
  }
//************************************************************************************************

  /*
  if (!getFix()) {
        //спутники есть и данные GPS надо сохранить в 24с
        if (address < maxAddr24) {
          ee24c.i2c_eeprom_write_page(EEPROM_ADDRESS, address, chGpsLint, sizestr);  //
//          eeprom.writeBytes(address, sizestr, chGpsLint);    //пишем строку GPS в EEPROM
          address += sizestr; // было sizestr;                      //сдвигаем адрес на пустое место
          // "под размер страницы" - чтоб не было ложной перезаписи данных
        }
     }
  */
  if (ignitON == 1)  {
    //после включения запала - шлем СМС периодически
        if (millis() > var_TimeSMS) {
            var_TimeSMS = millis() + TimeSMS;  //время отправки следующей СМС после вкл игнитора
            digitalWrite(LED_PIN, HIGH);

            // Get a fix and write it to global variable
            typeSMS = getFix();

            valSost = 3000;  //состояние отправки СМС (вкл игнитор)
            sendSms(valAlt, MY_NUMBER_WITH_PLUS);

            digitalWrite(LED_PIN, LOW);
        }
  }


//  addr = addr + 8;
  addr = addr + 2;   //в 470 пишем в Мегу только давление
  if (addr >= maxAddr)  {
   // запись завершена - фиксируем конечные данные
     valAlt = BMP280_readTemperature() +2000; //температура с BMP280 на 20 градусов выше и в 100 раз больше
     EEPROM.write(a_temp_fin, lowByte(valAlt));
     EEPROM.write(a_temp_fin+1, highByte(valAlt));
     valAlt = readVcc();                //питание устройства VCC(5V)    002
     EEPROM.write(a_vcc_mc_fin, lowByte(valAlt));
     EEPROM.write(a_vcc_mc_fin+1, highByte(valAlt));

//     voltage = analogRead(Vacb) * 2;      //напр. аккумулятора (там делитель  на 2)
//     voltage = (voltage * valAlt) / 1024;        //пересчет в мВ с учетом питания МК
//     H0 = voltage;
//     EEPROM.write(a_vcc_acc_fin, lowByte(H0));
//     EEPROM.write(a_vcc_acc_fin+1, highByte(H0));

     /*
  // читаем "сырые" данные с акселерометра (двухбайтовые)
  raw = accelerometer.readRaw();
  // запишем состояние "на старте"
  //Serial.print(raw.XAxis); 
  Atmp = raw.XAxis;
  EEPROM.write(a_ax_fin, lowByte(Atmp));
  EEPROM.write(a_ax_fin+1, highByte(Atmp));
  //Serial.print(raw.YAxis);
  Atmp = raw.YAxis;
  EEPROM.write(a_ay_fin, lowByte(Atmp));
  EEPROM.write(a_ay_fin+1, highByte(Atmp));
  //Serial.print(raw.ZAxis); 
  Atmp = raw.ZAxis;
  EEPROM.write(a_az_fin, lowByte(Atmp));
  EEPROM.write(a_az_fin+1, highByte(Atmp));
*/

     //память заполнена - светодиод  ГАСИМ
     digitalWrite(led, LOW);
     //отключаем выводы BMP280
     pinMode(_cs, INPUT);
     pinMode(_sck, INPUT);
     pinMode(_mosi, INPUT);
     pinMode(_miso, INPUT);
     digitalWrite(_vcc, LOW); //снимем питание
     pinMode(_vcc, INPUT);
     pinMode(_gnd, INPUT);
				//выставим флаги полета
                //признак старта=1, признак выброса парашюта=1, признак прошедшего полета=1
				resetFlags(1,1,1);            //установить все признаки
       
  digitalWrite(LED_PIN, HIGH);

  // Get a fix and write it to global variable
  typeSMS = getFix();

//Высота  (давление)
  valAlt = readPressure();
  valSost = 2000;  //состояние отправки СМС (память "забита")
  sendSms(valAlt, MY_NUMBER_WITH_PLUS);

  digitalWrite(LED_PIN, LOW);

             //тут надо периодическую отправку СМС          
             for (int _sms=0; _sms < 8; _sms++) { //примерно 2 мин шлем СМС (15 сек)

                  newDelay(10000);  //пауза 10 сек
                  // треск 5 сек
                  for (int z = 0; z < 30; z++) {
                  Beep60();
                  delay(100);      
                       }
             // отправка  СМС
             digitalWrite(LED_PIN, HIGH);

             // Get a fix and write it to global variable
             for (int stl = 1; stl < 5; stl++) {
                   if (getFix()) break;  //ждем координаты
                   newDelay(200);  //пауза 
                  }
                  
             valAlt = readPressure();    //Высота  (давление)
//             valAlt = maxAlt;            // max высота (миним.давление)
             valSost = 4000;  //состояние отправки СМС 
             sendSms(valAlt, MY_NUMBER_WITH_PLUS);

             digitalWrite(LED_PIN, LOW);
             }
      stopProc(6);              // останов с признаком "запись в EEPROM завершена"            
  }
    
  while ((tTek + tPress) > millis()) {} 
}

/*
разбор строки NMEA GPGGA на отдельные переменные
$GPGGA,073847.00,5610.83240,N,04045.18088,E,1,04,2.71,115.2,M,10.1,M,,*57
сразу заполняет разряды буфера 4..19
*/

boolean parsingGPGGA(byte* gpsBuffer, char* time1) {

    int i = 0;
    int j = 0;

    if (gpsBuffer[i] == '$') {  //проверяем на GPGGA
        i++;
        j = 0;
        while (gpsBuffer[i] != ',') {
            time1[j++] = gpsBuffer[i++];
        }
//       time1[j] = '\0';
     if (time1[2] != 'G') {return false;}
     if (time1[3] != 'G') {return false;}
     if (time1[4] != 'A') {return false;}
    }

    //собираем время - разобъем на две части: часы-минуты и секунды-сотые, чтоб упаковать в два двухбайтовых числа
    if (gpsBuffer[i] == ',') {
        i++;
        j = 0;
        while (gpsBuffer[i] != '.') {
            time1[j++] = gpsBuffer[i++];
        }
       time1[j-2] = '\0'; //оставим только часы-минуты
       _tmpW = atof(time1);   //и сразу поместим их в выходной буфер
       chGpsLint[4+1] = highByte(_tmpW);
       chGpsLint[4+0] = lowByte(_tmpW);
        i -= 2;   //вернемся к секундам
        j = 0;
        while (gpsBuffer[i] != '.') {
            time1[j++] = gpsBuffer[i++];
        }
        i++;
    //десятые и сотые также добавляем в строку времени (без точки)
         while (gpsBuffer[i] != ',') {
            time1[j++] = gpsBuffer[i++];
        }
       time1[j] = '\0';
       _tmpW = atof(time1);   //и сразу поместим их в выходной буфер
       chGpsLint[4+3] = highByte(_tmpW);
       chGpsLint[4+2] = lowByte(_tmpW);
     
    } else return false;

    // запись данных широты
    if (gpsBuffer[i] == ',') {
        i++;
        j = 0;
        while (gpsBuffer[i] != '.') {
            time1[j++] = gpsBuffer[i++];
        }
        time1[j] = '\0';      //сначала градусы и целые минуты - в двухбайтовое число
        _tmpW = atof(time1);   //и сразу поместим их в выходной буфер
        chGpsLint[8+1] = highByte(_tmpW);
        chGpsLint[8+0] = lowByte(_tmpW);

        i++;  //без всяких точек - в одну строку (пять знаков после запятой)
        j = 0;
        while (gpsBuffer[i] != ',') {
            time1[j++] = gpsBuffer[i++];
        }
        time1[j-1] = '\0';     //пятую цифру выбрасываем (это 2 см)
        _tmpW = atof(time1);   //и сразу поместим их в выходной буфер
        chGpsLint[8+3] = highByte(_tmpW);
        chGpsLint[8+2] = lowByte(_tmpW);

        i++;   //букву N(S) пропускаем - ограничимся северным полушарием
        while (gpsBuffer[i] != ',') {
            i++;
        }
    }  else return false;

    // запись данных долготы
    if (gpsBuffer[i] == ',') {
        i++;
        j = 0;
        while (gpsBuffer[i] != '.') {
            time1[j++] = gpsBuffer[i++];
        }
        time1[j] = '\0';      //сначала градусы и целые минуты - в двухбайтовое число
        _tmpW = atof(time1);   //и сразу поместим их в выходной буфер
        chGpsLint[12+1] = highByte(_tmpW);
        chGpsLint[12+0] = lowByte(_tmpW);

        i++;   //без всяких точек - в одну строку (пять знаков после запятой)
        j = 0;
        while (gpsBuffer[i] != ',') {
            time1[j++] = gpsBuffer[i++];
        }
        time1[j-1] = '\0';     //пятую цифру выбрасываем (это 2 см)
        _tmpW = atof(time1);   //и сразу поместим их в выходной буфер
        chGpsLint[12+3] = highByte(_tmpW);
        chGpsLint[12+2] = lowByte(_tmpW);

        i++;   //букву E(W) пропускаем - ограничимся восточной долготой
        while (gpsBuffer[i] != ',') {
            i++;
        }
    }   else return false;

    // состояние GPS и число спутников - в одну переменную: "107"  + игнитор
    if (gpsBuffer[i] == ',') {
        i++;
        time1[0] = gpsBuffer[i];
        i++;
    if (gpsBuffer[i] == ',') {
        i++;
        j = 1;
        while (gpsBuffer[i] != ',') {
        time1[j++] = gpsBuffer[i++];
        }
        time1[j] = '\0';
        _tmpW = atof(time1);   //и сразу поместим их в выходной буфер
        _tmpW += ignitON * 1000;  //добавим состояние игнитора четвертым знаком
        chGpsLint[16+1] = highByte(_tmpW);
        chGpsLint[16+0] = lowByte(_tmpW);
    }
    }  else return false;

    // пропуск данных о точности в горизонтали
    if (gpsBuffer[i] == ',') {
        i++;
        while (gpsBuffer[i] != ',') {
            i++;
        }
    }
    // запись данных высоты
    if (gpsBuffer[i] == ',') {
        i++;
        j = 0;
        while (gpsBuffer[i] != '.') {
            time1[j++] = gpsBuffer[i++];
        }
        i++;   //без всяких точек - в одну строку (один знак после запятой)
        while (gpsBuffer[i] != ',') {
            time1[j++] = gpsBuffer[i++];
        }
        time1[j] = '\0';
        _tmpW = atof(time1);   //и сразу поместим ее в выходной буфер
        chGpsLint[16+3] = highByte(_tmpW);
        chGpsLint[16+2] = lowByte(_tmpW);
    }   else return false;
    return true;

}

//занести данные о текущем адресе, давлении и напряжении в буфер вывода
// addr, valAlt, readVcc()
void APVtoGpsLint()  {
                chGpsLint[0] = 0;
                chGpsLint[1] = 0;
                chGpsLint[3] = highByte(addr);  //адрес меги
                chGpsLint[2] = lowByte(addr);
                
                // в valAlt значение давления
                chGpsLint[20+3] = highByte(valAlt);
                chGpsLint[20+2] = lowByte(valAlt);
                valAlt = readVcc();                //питание устройства VCC(5V)    
                chGpsLint[20+1] = highByte(valAlt);
                chGpsLint[20+0] = lowByte(valAlt);

                // температура
                valAlt = BMP280_readTemperature() +2000; //температура с BMP280    на 20 градусов выше и в 100 раз больше
                chGpsLint[24+1] = highByte(valAlt);
                chGpsLint[24+0] = lowByte(valAlt);
  /*              
                // ускорение по трем осям
                // читаем "сырые" данные с акселерометра (двухбайтовые)
                raw = accelerometer.readRaw();
                //Serial.print(raw.XAxis); 
                //Serial.print(raw.YAxis);
                //Serial.print(raw.ZAxis); 
                //ускорение по осям
                Atmp = raw.XAxis;
                chGpsLint[26+1] = highByte(Atmp);  //Х
                chGpsLint[26+0] = lowByte(Atmp);
                Atmp = raw.YAxis;
                chGpsLint[28+1] = highByte(Atmp);  //Y
                chGpsLint[28+0] = lowByte(Atmp);
                Atmp = raw.ZAxis;
                chGpsLint[30+1] = highByte(Atmp);  //Z
                chGpsLint[30+0] = lowByte(Atmp);
                */
}

// проверка корректности данных GPS
// false - ОК, спутники есть, данные корректны
// true - спутников нет (время проверки TimeSat)
bool getFix() {
                APVtoGpsLint();    //дополняем адресом, давлением и напряжением
                return false;
                /*
  bool noFix = true;
  
  //return noFix;     //308c
  
  if (Serial.available()) {                           
  //что-то в буфере есть - обрабатываем и принимаем как 0,5 сек отсчет (Neo-6M настроен на 0,5 сек)
  // Считываем 128 символов или до конца строки.
      Serial.setTimeout(200);  //ожидание байта перед выходом, если его нет
      btReaded = Serial.readBytesUntil('\n', chGpsLint, 128);
      if (btReaded > 48) {                  // Данные актуальны, длина строки от GPS-модуля.
           // тут надо подготовить все данные от GPS
           if (parsingGPGGA(chGpsLint, time1)) {  //заполняем массив
                APVtoGpsLint();    //дополняем адресом, давлением и напряжением

                return false;
                }
           }
//      else return noFix;  //считаем, что спутников нет
      }
//    else return noFix;  //данных GPS нет
*/
  /*
  long int timeWait = millis() + TimeSat;
  while (noFix && (timeWait > millis()))
  {
    while (gps.available( gpsPort )) {
      fix = gps.read();
      if (fix.valid.location && fix.valid.altitude
#ifdef SEND_GPS_TIMESTAMP
          && fix.valid.date && fix.valid.time
#endif
         ) {
           noFix = false;
           break;
      } 
    }
  }
  */
//  noFix = false;
//  return noFix;
  
}
/*
void sendAndWait(String cmd, String resp) {
  bool gotString = false;
  m590Serial.setTimeout(200);
  m590Flush();

  while (!gotString) {
    m590Serial.println(cmd);
    gotString = checkString(m590Serial.readString(), resp);
  }
}

// ждем вызов или окончание периода автоотправки СМС
void waitFor2Strings(String in1, String in2) {
  bool gotStrings = false;
  m590Serial.setTimeout(200);
  String val;

  while (!gotStrings) {
    
//    putToEE();    // выполняем замер и запись в ЕЕ
    
    val = m590Serial.readString();
    gotStrings = checkString(val, in1) &&
                 checkString(val, in2);
    if (gotStrings) {
      AutoSMS = !AutoSMS;
      valSost = 2000;   // состояние "принят вызов, вкл затвора"
      }
    else {
      if (AutoSMS) {
        gotStrings = (var_TimeSMS + TimeSMS) < millis();
        if (gotStrings) valSost = 2500;   // состояние "СМС по времени, вкл затвора"
        }
      }          
  }  */
/*
  while (!gotStrings && (var_TimeSMS + TimeSMS) > millis()) {
    val = m590Serial.readString();
    gotStrings = checkString(val, in1) &&
                 checkString(val, in2);
  }
*/
//}
/*
void waitForString(String in) {
  bool gotString = false;
  m590Serial.setTimeout(200);
  m590Flush();

  while (!gotString) {
    gotString = checkString(m590Serial.readString(), in);
  }
}

bool checkString(String val, String in) {
  return (val.indexOf(in) > -1);
}
*/
//void sendSms(gps_fix & fix, String number)
void sendSms(long int ff, String number)
{
  return;
  NumSMS ++;     // порядковый номер СМС
  
//  m590Flush();   // принимаем все, что идет с GSM

/*  
#ifdef SEND_SIM_BALANCE
  String b = getBalance();
#endif
*/
/*
  // Wait for prompt >
  sendAndWait("AT+CMGS=\"" + number + "\"", ">");

  m590Serial.print(NumSMS,0);             // порядковый номер
  m590Serial.print(' ');
  if (AutoSMS) {m590Serial.print('A');}   // автоотправка СМС
  else {m590Serial.print('S');}           // стоп автоотправки СМС
  m590Serial.print(' ');

  int _indx = 16;

  int _ii1 = int(byte(chGpsLint[_indx+1]) *256) + byte(chGpsLint[_indx]);
  float _grad = float(_ii1)/1000;
  m590Serial.print( _grad, 3 );           // до запятой игнитор, после - спутники
  m590Serial.print('S');                  // в satelites
  m590Serial.print(' ');
  
  _ii1 = int(byte(chGpsLint[_indx+3]) *256) + byte(chGpsLint[_indx+2]);
  _grad = float(_ii1)/10;
  m590Serial.print( _grad, 2 );           // высота
  m590Serial.print('M');                  // в метрах
  m590Serial.print(' ');

  //текущее давление КПа (из переменной-параметра)
  _grad = float(ff + 80000)/1000;
  m590Serial.print( _grad, 3 );           // давление
  m590Serial.print('K');                  // в КПа
  m590Serial.print(' ');
*/
/* 
  if (digitalRead(InPort) == HIGH) {
    m590Serial.print('H');}               // на входе "1"
  else {m590Serial.print('L');}           // "0"
  m590Serial.print(' ');

  if (digitalRead(InPort2) == HIGH) {
    m590Serial.print('H');}               // на входе "1"
  else {m590Serial.print('L');}           // "0"
  m590Serial.print(' ');

  float VCC = (float)readVcc()/1000.0;
  m590Serial.print(VCC, 2);               // напряжение питания
  m590Serial.print('V');                  // в вольтах
  m590Serial.print(' ');
  
  if (!typeSMS) {
#ifdef SEND_GPS_TIMESTAMP
  m590Serial.print(fix.dateTime.year);
  m590Serial.print('-');
  m590Serial.print(fix.dateTime.month);
  m590Serial.print('-');
  m590Serial.print(fix.dateTime.date);
  m590Serial.print(',');
  m590Serial.print(fix.dateTime.hours);
  m590Serial.print(':');
  m590Serial.print(fix.dateTime.minutes);
  m590Serial.print(':');
  m590Serial.print(fix.dateTime.seconds);
  sendAndWait(" UTC ", ">");
#endif

  // Send maps.google.com/blabla
  m590Serial.print(link);

  // Lat + lon from chGpsLint
  _indx = 8;
  _ii1 = int(byte(chGpsLint[_indx+1]) *256) + byte(chGpsLint[_indx+0]);
  int _intgrad = int(_ii1/100);
  int _intmin = _ii1 - _intgrad*100;
  int _ii2 = int(byte(chGpsLint[_indx+3]) *256) + byte(chGpsLint[_indx+2]);
  float _drobmin = float(_ii2)/10000;
  _grad = _intgrad + float((_intmin + _drobmin)/60);
  m590Serial.print( _grad, 6 );  //56.0000   fix.latitude()
  m590Serial.print( ',' );
 
  _indx = 12;
  _ii1 = int(byte(chGpsLint[_indx+1]) *256) + byte(chGpsLint[_indx+0]);
  _intgrad = int(_ii1/100);
  _intmin = _ii1 - _intgrad*100;
  _ii2 = int(byte(chGpsLint[_indx+3]) *256) + byte(chGpsLint[_indx+2]);
  _drobmin = float(_ii2)/10000;
  _grad = _intgrad + float((_intmin + _drobmin)/60);
  m590Serial.print( _grad, 6 ); //46.0000   fix.longitude()
  

#ifdef SEND_SIM_BALANCE
  sendAndWait("", ">");
  m590Serial.print("Balance is ");
  m590Serial.print(b);
  m590Serial.print(" RUR.");
#endif

  // End character
  m590Serial.print((char)0x1A);
  */
}

void newDelay(int ms) {
  delay(ms);
  return;
//  m590Serial.setTimeout(ms);
//  m590Serial.readString();
}
/*
void m590Flush() {
  while (m590Serial.available()) {
    m590Serial.read();
  }
}
*/
void GPSFlush() {
  while (Serial.available()) {
    Serial.read();
  }
}
